unit test;

interface

uses
  Windows, SysUtils, Forms, Spin, ExtCtrls, StdCtrls, Controls, Classes;

type
  TMainForm = class(TForm)
    StartButton: TButton;
    Panel1: TPanel;
    NumSpinEdit: TSpinEdit;
    ShowCheckBox: TCheckBox;
    Label1: TLabel;
    DisableCheckBox: TCheckBox;
    procedure StartButtonClick(Sender: TObject);
    procedure ButtonClick(Sender: TObject);
    procedure TestOver(won : boolean);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    Buttons : array of TButton; // number buttons
    NextN : Integer; // next number to press for correct sequence
    NumButtons : Byte; // number of number buttons
    StartTime : Real; // timer start time
    First : boolean; // first game flag
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

// start game
procedure TMainForm.StartButtonClick(Sender: TObject);
Var
  Index, i, NumPos : Byte;
  Positions : array of TPoint;
begin
  Label1.Caption := ' ';
  StartButton.Enabled := False;

  // destroy buttons from previous game
  if not First then for i := 0 to NumButtons - 1 do Buttons[i].Free; 

  if NumSpinEdit.Text = '' then NumSpinEdit.Text := '2';
  NumButtons := NumSpinEdit.Value; // numbers of buttons to show

  First := False;
  SetLength(Buttons, NumButtons);

  // create buttons
  for Index := 0 to NumButtons - 1 do begin
    Buttons[Index] := TButton.Create(Self);
    with Buttons[Index] do begin
      Parent := Panel1;
      OnClick := ButtonClick; // click event
      Tag := Index; // to back reference the button
      Caption := IntToStr(Index + 1); // display number on button
      Height := 49;
      Width := 49;
      Font.Name := 'MS Sans Serif';
      Font.Size := 24;
      Visible := True;
      Enabled := True;
    end;
  end;

  // create a list of all possible button positions
  NumPos := 117;
  SetLength(Positions, NumPos);
  for Index := 0 to 12 do for i := 0 to 8 do
    Positions[Index*9+i] := Point(Index, i);

  // set buttons to unique random positions
  for Index := 0 to NumButtons - 1 do begin
    // select a position from the list at random
    i := Random(NumPos);
    Buttons[Index].Left := Positions[i].X * 48;
    Buttons[Index].Top  := Positions[i].Y * 48;
    // remove this position from the list
    Dec(NumPos);
    Positions[i] := Positions[NumPos]; // replace with last position
    SetLength(Positions, NumPos);
  end;

  NextN := 0;

  StartTime := Now; // start timer
end;

// number button pressed
procedure TMainForm.ButtonClick(Sender: TObject);
Var
  Num, i : byte;
begin
  Num := TButton(Sender).Tag; // get the number of the button that was pressed

  // test that correct button was pressed
  if NextN <> Num then begin
    TestOver(False);
    Exit;
  end;

  Inc(NextN); // increment the next required number

  // clear the numbers if first button
  if (NextN = 1) and not ShowCheckBox.Checked then
    for i := 0 to NumButtons - 1 do Buttons[i].Caption := ' ';

  // check for all buttons pressed - win
  if Num = NumButtons - 1 then TestOver(True) else
    if DisableCheckBox.Checked then TButton(Sender).Enabled := False; // disable this button
end;

// game over
procedure TMainForm.TestOver(won : boolean);
Var
  i : byte;
begin
  // reset buttons
  for i := 0 to NumButtons - 1 do begin
    Buttons[i].Caption := IntToStr(i + 1);
    Buttons[i].Enabled := False;
  end;

  if not won then begin
    Beep;
    Label1.Caption := 'Failed';
  end else // show time
    Label1.Caption := FloatToStr(Round((Now-StartTime)*8640000)/100) + ' s';

  StartButton.Enabled := True;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  First := True;
end;

end.
